/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dict
 *
 * The Initial Developer of the Original Code is David Costanzo.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// The preferences that we handle
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";

function DictServer (name, port, description) {
    this.name        = name;
    this.port        = port;
    this.description = description;
}

// The list of valid DICT servers
var DICT_SERVERS = [
    new DictServer(
        'dega.cs.unc.edu',
        '2628',
        'To and from lojban and a number of other languages.'),
    new DictServer(
        'dict.arabeyes.org',
        '2628',
        'English-Arabic, extensible'),
    new DictServer(
        'dict.netground.com',
        '2628',
        'Standard and freedict databases, GCIDE, Netground ISW\'s Dictionary'),
    new DictServer(
        'dict.org',
        '2628',
        'The canonical dictd server, contains mainly databases related to English language'),
    new DictServer(
        'dict.rucus.ru.ac.za',
        '2628',
        'Standard dictd server with eleven English databases.'),
    new DictServer(
        'dict.tu-chemnitz.de',
        '2628',
        'Standard DICT server with 9 databases (devils, elements, foldoc, gazetteer, gcide, jargon, vera, web1913, Eastons, Hitchcock\'s, CIA World Factbook 1995, CIA World Factbook 2002 and wn)'),
    new DictServer(
        'dict.tugraz.at',
        '2628',
        'standard and freedict databases'),
    new DictServer(
        'dict.uni-leipzig.de',
        '2628',
        'Khasi-German, German-Khasi (1000 headwords), English-Hindi (25645 headwords)'),
    new DictServer(
        'kassiopeia.juls.savba.sk',
        '2628',
        'Many different dictionaries (en, slovak, hu, ru, bulgarian, serbian, german, latin, ...)'),
    new DictServer(
        'localhost',
        '2628',
        'The DICT server that is running on your local machine.'),
    new DictServer(
        'madison.onespeeddave.com',
        '2628',
        'Web interface offers cross-referenced definitions from 11 standard databases, automatic spelling correction, regular expressions, and also supports queries via http://[query].dict.die.net.'),
    new DictServer(
        'mova.org',
        '2628',
        'A collection of Slavic dictionaries'),
    new DictServer(
        'slovnik.zcu.cz',
        '2628',
        'chinese-english, moby thesaurus, standard databases'),
    new DictServer(
        'test.dict.org',
        '2628',
        'Standard databases'),
    new DictServer(
        'www.okpos.com',
        '2628',
        'Standard databases'),
];


// handler for when a new DICT server is selected in the list box
function newDictServerSelected() {
    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);


    var descriptionElement = document.getElementById("description"); 

    // remove any exising description
    while (descriptionElement.firstChild) {
        descriptionElement.removeChild(descriptionElement.firstChild);
    }

    var listbox = document.getElementById("dictd-servers");

    var serverID = listbox.selectedIndex;

    // Update the description text to be match the description of the
    // newly selected server.  Also update the hidden preference elements.
    if (serverID < 0 || DICT_SERVERS.length <= serverID) {

        // The selected server is not in our stock list of suggested
        // servers.  Do not choose one by default and modify the user's
        // preferences, because the user may have hacked their prefs.js
        // file to point to a server that is not on our hard-coded list.

        // add the new description
        descriptionText = 'Please select a server';
        var descriptionTextNode = document.createTextNode(descriptionText);
        descriptionElement.appendChild(descriptionTextNode);
    }
    else {
        var server = DICT_SERVERS[serverID];

        // update the preferences in our hidden preference elements
        var defaultServerName = document.getElementById('defaultservername');
        defaultServerName.setAttribute('value', server.name);

        var defaultServerPort = document.getElementById('defaultserverport');
        defaultServerPort.setAttribute('value', server.port);

        // add the new description
        var descriptionTextNode = document.createTextNode(server.description);
        descriptionElement.appendChild(descriptionTextNode);
    }
}

// Get the default server name, or use a default value
// Does not throw an exception.
function dictGetDefaultServerName() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        return pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
function dictGetDefaultServerPort() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERPORT);
     }
     catch (e) {
     }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}

// Set the server name preference.  Throw an exception if it cannot be set
function dictSetDefaultServerName(servername) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    
    pref.setCharPref(PREFNAME_DEFAULTSERVERNAME, servername);
}

// Set a server port preference.  Throw an exception if it cannot be set
function dictSetDefaultServerPort(serverport) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    
    try {
        pref.setCharPref(PREFNAME_DEFAULTSERVERPORT, serverport);
    }
    catch (e) {
    }

    // maybe it's stored as an int
    pref.setIntPref(PREFNAME_DEFAULTSERVERPORT, serverport);
}


// best-effort to set the preferences to known good values if they cannot 
// be read.  This does not throw an exception.
function dictSetDefaultPreferences() {

    try {
        var servername = dictGetDefaultServerName();
        dictSetDefaultServerName(servername);
    }
    catch (e) {
        // ignore errors
    }

    try {
        var serverport = dictGetDefaultServerPort();
        dictSetDefaultServerPort(serverport);
    }
    catch (e) {
        // ignore errors
    }
}



// This gets called automatically from Mozilla Application Suite
function Startup() {

    // set some default preferences, in case none exist
    dictSetDefaultPreferences();

    var listbox = document.getElementById("dictd-servers");

    for (dictserver in DICT_SERVERS) {

        // append this server to the list box
        var listitem = document.createElement('listitem');
        listitem.setAttribute('value', dictserver);
        listitem.setAttribute('label', DICT_SERVERS[dictserver].name);

        listbox.appendChild(listitem);
    }


    // Try to find an index in DICT_SERVERS that corresponds to the selected 
    // default server so that we can select it.
    var defaultServerIndex = 0;

    var defaultServerName = dictGetDefaultServerName();
    var defaultServerPort = dictGetDefaultServerPort();
        
    var currentServerIndex;
    for (currentServerIndex in DICT_SERVERS) {
        if (DICT_SERVERS[currentServerIndex].name == defaultServerName &&
            DICT_SERVERS[currentServerIndex].port == defaultServerPort) {
            defaultServerIndex = currentServerIndex;
            break;
        }
    }

    listbox.selectedIndex = defaultServerIndex;
}



function dictPreferencesInit() {
    // Manually call the Startup since Mozilla Firefox doesn't call it
    Startup();
}


function dictPreferencesSave() {

    // Manually save prefs; Mozilla Firefox doesn't do it
    try {

        // set the server name
        var defaultServerName = document.getElementById("defaultservername");
        dictSetDefaultServerName(defaultServerName.value);

        // set the server port
        var defaultServerPort = document.getElementById("defaultserverport");
        dictSetDefaultServerPort(defaultServerPort.value);
    } 
    catch(e) {

        // HACK:
        // Switch out this alert because it causes problems on Firefox 0.8.
        // For some reason, setting the defaultserverport raises an exeception.
        // We don't really need to do this because all the dict servers 
        // that we offer listen on port 2628, which is the port that 
        // will be used if the defaultserverport option is corrupt.
        //
        // If we ever want to select a server that listens on a non-default
        // port, this will silently fail, and the source of the problem
        // could be hard to track down.
        if (0) {
            alert(
                'Unable to save preferences.\n' + 
                'If the problem persists, please file a bug report.');
        }
    }
}
